############## Python-OpenCV Playing Card Detector ###############
#
# Author: Evan Juras
# Date: 9/5/17
# Description: Python script to detect and identify playing cards
# from a PiCamera video feed.
#

# Import necessary packages
import cv2
import numpy as np
import time
import os
import Cards
import VideoStream
import Game
import RPi.GPIO as GPIO


### ---- INITIALIZATION ---- ###
# Define constants and initialize variables

## Camera settings
IM_WIDTH = 1280
IM_HEIGHT = 720 
FRAME_RATE = 10

## Initialize calculated frame rate because it's calculated AFTER the first time it's displayed

## Define font to use
font = cv2.FONT_HERSHEY_SIMPLEX

# Initialize camera object and video feed from the camera. The video stream is set up
# as a seperate thread that constantly grabs frames from the camera feed. 
# See VideoStream.py for VideoStream class definition
## IF USING USB CAMERA INSTEAD OF PICAMERA,
## CHANGE THE THIRD ARGUMENT FROM 1 TO 2 IN THE FOLLOWING LINE:
#videostream = VideoStream.VideoStream((IM_WIDTH,IM_HEIGHT),FRAME_RATE,2,0).start()
#time.sleep(1) # Give the camera time to warm up
print("Starting")


### ---- MAIN LOOP ---- ###
# The main loop repeatedly grabs frames from the video stream
# and processes them to find and identify playing cards.

cam_quit = 0 # Loop control variable

# Begin capturing frames
while cam_quit == 0:
    # Grab frame from video stream
    cam = cv2.VideoCapture(0)
    cam.set(cv2.CAP_PROP_FRAME_WIDTH, 1280)
    cam.set(cv2.CAP_PROP_FRAME_HEIGHT, 720)
    
    ret, image = cam.read()
    
    cam.release()
    #print("new Frame")
    # Start timer (for calculating frame rate)
    #t1 = cv2.getTickCount()

    # Pre-process camera image (gray, blur, and threshold it)
    pre_proc = Cards.preprocess_image(image)
    
    # Find and sort the contours of all cards in the image (query cards)
    cnts_sort, cnt_is_card = Cards.find_cards(pre_proc)

    # If there are no contours, do nothing
    if len(cnts_sort) != 0:
        print("found Cards")
        im2 = image.copy()
        cv2.drawContours(im2,cnts_sort,-1, (255,0,0), 2)
        cv2.imshow("cnts", im2)
        # Initialize a new "cards" list to assign the card objects.
        # k indexes the newly made array of cards.
        cards = []
        hand = []
        k = 0
        turned = 0
        # For each contour detected:
        print(Game.playstack.rank_name)
        for i in range(len(cnts_sort)):
            if (cnt_is_card[i] == 1):
                # Create a card object from the contour and append it to the list of cards.
                # preprocess_card function takes the card contour and contour and
                # determines the cards properties (corner points, etc). It generates a
                # flattened 200x300 image of the card, and isolates the card's
                # suit and rank from the image.
                cards.append(Cards.preprocess_card(cnts_sort[i],image))         #cards ist eine Liste aus qCards

                # Find the best rank and suit match for the card.
                #cards[k].best_rank_match,cards[k].best_suit_match,cards[k].rank_diff,cards[k].suit_diff = Cards.match_card(cards[k],train_ranks,train_suits)

                # Draw center point and match result on the image.
                #image = Cards.draw_results(image, cards[k])

                if (cards[k].center[0] < 1000 or cards[k].center[1] > 200):
                    hand.append(cards[k])
                else:
                    if (Game.playstack.rankval != cards[k].rankval or Game.playstack.suitval != cards[k].suitval):
                        turned = 1
                        print("turned")
                    Game.playstack = cards[k]
                k = k + 1
        #print("Finished reading Hand")
        if turned == 1:
            Game.PlayCard(hand)
            turned = 0


        #for i in range(len(hand)):
         #   cv2.putText(image, (hand[i].rank_name + ' of'), (60, i * 40+20), font, 1, (0, 0, 0), 2, cv2.LINE_AA)
          #  cv2.putText(image, (hand[i].suit_name), (60, i * 40+40), font, 1, (0, 0, 0), 2, cv2.LINE_AA)

        # Draw card contours on image (have to do contours all at once or
        # they do not show up properly for some reason)
        if (len(cards) != 0):
            temp_cnts = []
            for i in range(len(cards)):
                temp_cnts.append(cards[i].contour)
            cv2.drawContours(image,temp_cnts, -1, (255,0,0), 2)

        
    # Draw framerate in the corner of the image. Framerate is calculated at the end of the main loop,
    # so the first time this runs, framerate will be shown as 0.
    #cv2.putText(image,"FPS: "+str(int(frame_rate_calc)),(10,26),font,0.7,(255,0,255),2,cv2.LINE_AA)

    # Finally, display the image with the identified cards!
    image = cv2.circle(image, (1000, 200),10, (255,0,0), 5)
    cv2.imshow("Card Detector",image)

    # Calculate framerate

    # Poll the keyboard. If 'q' is pressed, exit the main loop.
    key = cv2.waitKey(1) & 0xFF
    if key == ord("q"):
        cam_quit = 1
      
      



# Close all windows and close the PiCamera video stream.
cv2.destroyAllWindows()

